@@ -1,18 +1,18 @@
 /* Ordered {set,map} data type implemented by a binary tree.
-   Copyright (C) 2006-2007, 2009-2019 Free Software Foundation, Inc.
+   Copyright (C) 2006-2007, 2009-2022 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2006.
 
-   This program is free software: you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
-   (at your option) any later version.
+   This file is free software: you can redistribute it and/or modify
+   it under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of the
+   License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* A red-black tree is a binary tree where every node is colored black or
@@ -63,7 +63,7 @@ struct CONTAINER_IMPL
    this would exceed the address space of the machine.  */
 #define MAXHEIGHT 116
 
-/* Rotate left a subtree.
+/* Rotates left a subtree.
 
                          B                         D
                        /   \                     /   \
@@ -71,7 +71,7 @@ struct CONTAINER_IMPL
                             / \               / \
                            C   E             A   C
 
-   Change the tree structure, update the branch sizes.
+   Changes the tree structure, updates the branch sizes.
    The caller must update the colors and register D as child of its parent.  */
 static NODE_T
 rotate_left (NODE_T b_node, NODE_T d_node)
@@ -89,7 +89,7 @@ rotate_left (NODE_T b_node, NODE_T d_node)
   return d_node;
 }
 
-/* Rotate right a subtree.
+/* Rotates right a subtree.
 
                            D                     B
                          /   \                 /   \
@@ -97,7 +97,7 @@ rotate_left (NODE_T b_node, NODE_T d_node)
                       / \                           / \
                      A   C                         C   E
 
-   Change the tree structure, update the branch sizes.
+   Changes the tree structure, updates the branch sizes.
    The caller must update the colors and register B as child of its parent.  */
 static NODE_T
 rotate_right (NODE_T b_node, NODE_T d_node)
@@ -115,7 +115,7 @@ rotate_right (NODE_T b_node, NODE_T d_node)
   return b_node;
 }
 
-/* Ensure the tree is balanced, after an insertion operation.
+/* Ensures the tree is balanced, after an insertion operation.
    Also assigns node->color.
    parent is the given node's parent, known to be non-NULL.  */
 static void
@@ -239,7 +239,7 @@ rebalance_after_add (CONTAINER_T container, NODE_T node, NODE_T parent)
     }
 }
 
-/* Ensure the tree is balanced, after a deletion operation.
+/* Ensures the tree is balanced, after a deletion operation.
    CHILD was a grandchild of PARENT and is now its child.  Between them,
    a black node was removed.  CHILD is also black, or NULL.
    (CHILD can also be NULL.  But PARENT is non-NULL.)  */
@@ -565,19 +565,12 @@ gl_tree_nx_add_first (CONTAINER_T container, NODE_PAYLOAD_PARAMS)
   return new_node;
 }
 
-static NODE_T
-gl_tree_nx_add_before (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
+/* Adds the already allocated NEW_NODE to the tree, right before NODE.  */
+static void
+gl_tree_add_node_before (CONTAINER_T container, NODE_T node, NODE_T new_node)
 {
-  /* Create new node.  */
-  NODE_T new_node =
-    (struct NODE_IMPL *) malloc (sizeof (struct NODE_IMPL));
-
-  if (new_node == NULL)
-    return NULL;
-
   new_node->left = NULL;
   new_node->right = NULL;
-  NODE_PAYLOAD_ASSIGN(new_node)
 
   /* Add it to the tree.  */
   if (node->left == NULL)
@@ -594,11 +587,10 @@ gl_tree_nx_add_before (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
   rebalance_after_add (container, new_node, node);
 
   container->count++;
-  return new_node;
 }
 
 static NODE_T
-gl_tree_nx_add_after (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
+gl_tree_nx_add_before (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
 {
   /* Create new node.  */
   NODE_T new_node =
@@ -607,9 +599,18 @@ gl_tree_nx_add_after (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
   if (new_node == NULL)
     return NULL;
 
+  NODE_PAYLOAD_ASSIGN(new_node)
+
+  gl_tree_add_node_before (container, node, new_node);
+  return new_node;
+}
+
+/* Adds the already allocated NEW_NODE to the tree, right after NODE.  */
+static void
+gl_tree_add_node_after (CONTAINER_T container, NODE_T node, NODE_T new_node)
+{
   new_node->left = NULL;
   new_node->right = NULL;
-  NODE_PAYLOAD_ASSIGN(new_node)
 
   /* Add it to the tree.  */
   if (node->right == NULL)
@@ -626,11 +627,26 @@ gl_tree_nx_add_after (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
   rebalance_after_add (container, new_node, node);
 
   container->count++;
+}
+
+static NODE_T
+gl_tree_nx_add_after (CONTAINER_T container, NODE_T node, NODE_PAYLOAD_PARAMS)
+{
+  /* Create new node.  */
+  NODE_T new_node =
+    (struct NODE_IMPL *) malloc (sizeof (struct NODE_IMPL));
+
+  if (new_node == NULL)
+    return NULL;
+
+  NODE_PAYLOAD_ASSIGN(new_node)
+
+  gl_tree_add_node_after (container, node, new_node);
   return new_node;
 }
 
-static bool
-gl_tree_remove_node (CONTAINER_T container, NODE_T node)
+static void
+gl_tree_remove_node_no_free (CONTAINER_T container, NODE_T node)
 {
   NODE_T parent = node->parent;
 
@@ -749,7 +765,13 @@ gl_tree_remove_node (CONTAINER_T container, NODE_T node)
     }
 
   container->count--;
-  NODE_PAYLOAD_DISPOSE
+}
+
+static bool
+gl_tree_remove_node (CONTAINER_T container, NODE_T node)
+{
+  gl_tree_remove_node_no_free (container, node);
+  NODE_PAYLOAD_DISPOSE (container, node)
   free (node);
   return true;
 }
