@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "gl_list.h"
+#include "gl_map.h"
 
 struct binfmt {
     char *name;
@@ -36,9 +36,8 @@ struct binfmt {
 };
 
 struct binfmt *binfmt_load (const char *name, const char *filename, int quiet);
-struct binfmt *binfmt_new (const char *name, gl_list_t args);
+struct binfmt *binfmt_new (const char *name, gl_map_t args);
 int binfmt_write (const struct binfmt *binfmt, const char *filename);
 void binfmt_print (const struct binfmt *binfmt);
 int binfmt_equals (const struct binfmt *left, const struct binfmt *right);
 void binfmt_free (struct binfmt *binfmt);
-void kvhash_dispose_binfmt (void *data);
